
#
# This Program is to
# (1) Build groups in D_n, S_n, A_n
# (2) Calculate Product
# (3) Save the output into a .tex file
#
# by Wieslaw Krawcewicz, Zhichao Li
# on Thursday, April 24, 2014
# at Three Georges University, Yichang, Hubei, Chia
#

#
# Structure of Program
#
# Declare Constants
# Declare Global Variables
# Declare Functions
# Main Program
#

#
# to run this file, type the following in GAP:
# Read("ProductGroup20140724.gap");
#




#Read("AmalgaNew20140725_Leaf20140727.gap");




########## Declare Constants ##########

undefined_value := "Undefined";



########## Declare Global Variables ##########









########## Declare Global Functions ##########

minimum := function( a, b )

local
	c;
	
	if a < b then c := a; else c := b; fi;
	
	return c;

end;







########## Calculate n(L,H) number ###########################
#                                                            #
# L, H are conjugacy classes (L) and (H)                     #
#                                                            #
##############################################################

calculate_nLH_number := function( G )
	local	counter,
		i,j,k;
	

	Print("\n", "Calculating n(L,H) numbers ", "......", "\n");

	G.nLH_Table := [];
	for i in [1..Size(G.ccsl)] do 
		Add(G.nLH_Table, []);
		
		for j in [1..Size(G.ccsl)] do 
			Add(G.nLH_Table[i], []);
		od;
		
	od;
	
	for i in [1..Size(G.ccsl)] do # i is L here

		for j in [1..Size(G.ccsl)] do # j is H here		
			counter := 0;
			
			for k in G.ccsl[j].right_transversal_for_normalizer do
				if IsSubgroup( ConjugateGroup(G.ccsl[ j].representative, k ) , G.ccsl[i].representative   ) then
					counter := counter + 1;
				fi;			
			od;
			
			G.nLH_Table[i][j] := counter;
			
		od;

	od;
#	
#	for i in [1..Length(G.ccsl)] do
#		for j in [1..Length(G.ccsl)] do
#		
#			Print(G.nLH_Table[i][j], " ");
#		od;
#		Print("\n");
#	od;
#
	return;
end; # calculate_nLH_number := function()













##############################################################
#                                                            #
# calculate multiplication table for Burnside Ring           #
#                                                            #
##############################################################


calculate_multiplication_table := function( G )
	
local	
	counter,
	h,i,j,k,l,m,n;

	Print("\n", "Calculating Multiplication Table ", "......", "\n");

	G.Multiplication_Table := [];
	for i in [1..Size(G.ccsl)] do 
		Add(G.Multiplication_Table, []);
		
		for j in [1..Size(G.ccsl)] do 
			Add(G.Multiplication_Table[i], []);

			for k in [1..Size(G.ccsl)] do 
				Add(G.Multiplication_Table[i][j], []);
				
				G.Multiplication_Table[i][j][k] := 0;
			od;


		od;
		
	od;
	
	

	

	j := Size(G.ccsl);  # j is H
		while j>= 1 do

		k := Size(G.ccsl); # k is K
		while k>= 1 do
	
			i := Size(G.ccsl);     # i is L
			while i>= 1 do


				counter :=	G.nLH_Table[i][j] *
						G.ccsl[j].order_Weyl_group *
						G.nLH_Table[i][k] *
						G.ccsl[k].order_Weyl_group;
						
						
				
				l := Size(G.ccsl);  # l is L tilde
				while l> i do
				
					counter := counter - 
							(G.Multiplication_Table[j][k][l] * 
							G.nLH_Table[ i ] [ l ] * 
							G.ccsl[l].order_Weyl_group);
				
					l := l - 1;
				od;


				counter := counter / G.ccsl[i].order_Weyl_group;
	
				G.Multiplication_Table[j][k][i] := counter;
				
	
				i := i - 1;
			od;
	
	
			k := k -1 ;
		od;
	
	
		j := j-1;
	od;
		

	return;
end; # calculate_multiplication_table := function()








###################################################################
#                                                                 #
# Calculate Degrees of Basic Maps for Group G                     #
#                                                                 #
###################################################################

calculate_degrees_of_basic_maps := function( G )

local 
	representation,
	Representation_List,
	degree_component,
	counter,
	rlist,
	bb,
	a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z;

		

	Print( "\nCalculating degrees for basic maps ...... \n\n");

	rlist := IrreducibleRepresentations( G.group );

	G.Representation_List := [];

	for i in [1..Size(rlist)] do

		representation := rec
		(
			degree_component_list := undefined_value,
			degree_LaTeXName := undefined_value
		);
		
		representation.degree_component_list := [];
		
		degree_component := rec
		(
			dimension := undefined_value,
			group_id := Size(G.ccsl),
			degree_coefficient := undefined_value
		);

		#Print( "\n debug 1 \n" );		

		degree_component.dimension := 0;


		#Print( "\n H = ", G.ccsl[ degree_component.group_id ], " \n" );		

		for g in G.ccsl[ degree_component.group_id ].representative do

			#Print( "\n g = ", g, " \n" );		

			degree_component.dimension := degree_component.dimension + TraceMat( Image( rlist[i] , g ) );
		od;
		
		degree_component.dimension := degree_component.dimension / Order( G.ccsl[ degree_component.group_id ].representative );
		
		
				
		#Print( "\n debug 20 \n" );		

		if RemInt( degree_component.dimension, 2 ) = 0 
			then degree_component.degree_coefficient := 1;
			else degree_component.degree_coefficient := -1;
		fi;

		Add( representation.degree_component_list, degree_component );


		#Print( "\n debug 30 \n" );		

		j := Size(G.ccsl);
		while j > 1 do
			j := j - 1;

			degree_component := rec
			(
				dimension := undefined_value,
				group_id := j,
				degree_coefficient := undefined_value
			);
	
			degree_component.dimension := 0;
			for g in G.ccsl[ degree_component.group_id ].representative do
				degree_component.dimension := degree_component.dimension + TraceMat( Image( rlist[i] , g ) );
			od;

			degree_component.dimension := degree_component.dimension / Order( G.ccsl[ degree_component.group_id ].representative );


		
			bb := "true";
			
			for k in [1..Size(representation.degree_component_list)] do
				if 
					G.nLH_Table[ degree_component.group_id ][ representation.degree_component_list[k].group_id ] > 0 and
					degree_component.dimension <= representation.degree_component_list[k].dimension 
				then bb := "false"; break;
				fi;	
			od;	
			
			if bb = "false" then continue; fi;			



			if RemInt( degree_component.dimension, 2 ) = 0 
				then degree_component.degree_coefficient := 1;
				else degree_component.degree_coefficient := -1;
			fi;



			for k in [1..Size(representation.degree_component_list)] do
				
				degree_component.degree_coefficient :=
					degree_component.degree_coefficient -
					representation.degree_component_list[k].degree_coefficient *
					G.nLH_Table[ degree_component.group_id ][ representation.degree_component_list[k].group_id ] *
					G.ccsl[ representation.degree_component_list[k].group_id ].order_Weyl_group;
				
			od;	
			
			degree_component.degree_coefficient := degree_component.degree_coefficient / G.ccsl[ degree_component.group_id ].order_Weyl_group;


			Add( representation.degree_component_list, degree_component );
			
		od;


		
		
		representation.LaTeXName := "";
		
		for j in [1..Size(representation.degree_component_list)] do
			t := representation.degree_component_list[j].degree_coefficient;
			s := G.ccsl[ representation.degree_component_list[j].group_id ].LaTeXName;
			
			if t = 0 then continue; fi;

			if t < 0 then
				representation.LaTeXName := Concatenation( representation.LaTeXName, "-" );
				t := - t;
			elif j > 1 then
				representation.LaTeXName := Concatenation( representation.LaTeXName, "+" );
			fi;			
			
			if t > 1 then
				representation.LaTeXName := Concatenation( representation.LaTeXName, String(t) );
			fi;			
			
			representation.LaTeXName := Concatenation( representation.LaTeXName, "(", s, ")" );

		od;


		Add( G.Representation_List, representation );

	od;
	
	return;

end;  # calculate_degrees_of_basic_maps()










############################################
#                                          #
#          Create LaTeX file               #
#                                          #
############################################


create_LaTeX_file := function( G, print_multiplication_table, print_basic_degree )
	local	gsys, # LaTeX file
		conjugacy_class,
		representation,
		counter,
		rows_page,
		columns_page,
		left_column,
		top_row,
		#print_multiplication_table,
		a, b, c, d, e, f, g, h, i,j,k,l,m,n, o, p, q, r, s, t, u, v, w, x, y, z,
		ii, jj, kk, ll, mm, nn;


	if G.IsProductGroup = false then
		Print("\n", G.TextName, " is not a product group ...\n");	
	
		#stop;
	fi;


	Print("\n", "Creating LaTeX File ......", "\n");

	# assign the file name to the LeTeX file
	gsys := Concatenation( G.G1.TextName, "x", G.G2.TextName, ".tex" );

	gsys := OutputTextFile( gsys, false );
	SetPrintFormattingStatus( gsys, false );


	PrintTo(gsys,"\n","\\documentclass[11pt]{article}","\n\n");

	AppendTo(gsys,"\\usepackage{amsfonts}","\n");
	AppendTo(gsys,"\\usepackage{amsmath}","\n");
	AppendTo(gsys,"\\usepackage{amssymb}","\n");
	AppendTo(gsys,"\\usepackage{graphicx}","\n");
	AppendTo(gsys,"\\usepackage{pstricks}","\n");


	AppendTo(gsys,"\\def\\bb{\\mathbb}","\n");	
	AppendTo(gsys,"\\def\\ft{\\footnotesize}","\n");
	AppendTo(gsys,"\\def\\vp{\\varphi}","\n");
	AppendTo(gsys,"\\def\\tm{\\times}","\n");
	AppendTo(gsys,"\\def\\wt{\\widetilde}","\n");


	AppendTo(gsys,"\n");

	AppendTo(gsys,"\\begin{document}","\n");	


	AppendTo(gsys,"\\title { Description of Group $", G.LaTeXName, "$}  \n");
	AppendTo(gsys,"\\maketitle","\n");

	AppendTo(gsys,"\\begin{center}", "\n");
	AppendTo(gsys,"{\\large Subconjugacy of $", G.G1.LaTeXName, "$}\\\\[8pt]\n");

	AppendTo(gsys,"\\begin{tabular}{*{",2,"}{|c|}}\n\\hline\n");
	AppendTo(gsys,"\n");

	AppendTo(gsys,"Number"," & ","Subconjugacy", " \\cr");
	AppendTo(gsys,"\\hline\n");
	

	counter := 0;
	for conjugacy_class in G.G1.ccsl do
		counter := counter + 1;
		AppendTo(gsys, String(counter), " & ");
		AppendTo(gsys, "$(", conjugacy_class.LaTeXName, ")$ \\cr ");
		AppendTo(gsys, "\n");
	od;


	AppendTo(gsys,"\\hline");
	AppendTo(gsys,"\n\\end{tabular}\n\\end{center}\n");





	AppendTo(gsys, "\n", "\\newpage", "\n");


	AppendTo(gsys,"\\begin{center}", "\n");
	AppendTo(gsys,"{\\large Subconjugacy of $", G.G2.LaTeXName, "$}\\\\ [8pt]\n");

	AppendTo(gsys,"\\begin{tabular}{*{",2,"}{|c|}}\n\\hline\n");
	AppendTo(gsys,"\n");

	AppendTo(gsys,"Number"," & ","Subconjugacy", " \\cr");
	AppendTo(gsys,"\\hline\n");
	

	counter := 0;
	for conjugacy_class in G.G2.ccsl do
		counter := counter + 1;
		AppendTo(gsys, String(counter), " & ");
		AppendTo(gsys, "$(", conjugacy_class.LaTeXName, ")$ \\cr ");
		AppendTo(gsys, "\n");
	od;


	AppendTo(gsys,"\\hline");
	AppendTo(gsys,"\n\\end{tabular}\n\\end{center}\n");






	counter := 0;

	rows_page := 30;

	i := 0;

	while counter < Size( G.ccsl ) do

		i := i + 1;
		
		j := 0;
#
#		while 	(0 = counter) or
#				(
#				counter < Size( G.ccsl ) and 
#				G.ccsl[counter+1].HNumberOne = i
#				) 
#				do
#	


			if 	counter > 0 and 
				RemInt( counter, rows_page ) = 0 
			then

				AppendTo(gsys,"\\hline");
				AppendTo(gsys,"\n\\end{tabular}\n\\end{center}\n");

			fi;

			if RemInt( counter, rows_page ) = 0 then

				AppendTo(gsys, "\n", "\\newpage", "\n");


				AppendTo(gsys,"\\begin{center}{\\large Subconjugacy of $", G.G1.LaTeXName, " \\tm ", G.G2.LaTeXName, "$}\\\\[8pt]\n");

				AppendTo(gsys,"\\begin{tabular}{*{",6,"}{|c|}}\n\\hline\n");
				AppendTo(gsys,"\n");

				AppendTo(gsys, "Number"," & " );
				AppendTo(gsys, "$(H)$"," & " );
				AppendTo(gsys, "Ker($\\vp$)"," & " );
				AppendTo(gsys, "Ker($\\psi$)"," & " );
				AppendTo(gsys, "$N(H)$"," & " );
				AppendTo(gsys, "$|W(H)|$", " \\cr");
				AppendTo(gsys,"\\hline\n");
	
			fi;


	
			j := j + 1;	

			counter := counter + 1;

			Print( "\n counter = ", counter, "  i = ", i, "     j = ", j, "\n");	
	
			AppendTo(gsys, String(counter), " & ");
			
			AppendTo(gsys, "$(", G.ccsl[counter].LaTeXName, " )$ & ");

			AppendTo(gsys, "$ ", G.ccsl[counter].KernelPhi, " $ & ");

			AppendTo(gsys, "$ ", G.ccsl[counter].KernelPsi, " $ & ");

			AppendTo(gsys, "$ ", G.ccsl[counter].normalizer_name, " $ & ");

			AppendTo(gsys, String( Order( G.ccsl[counter].normalizer ) / Order( G.ccsl[counter].representative ) ), " \\cr ");


			
			AppendTo(gsys, "\n");




#
#		od;
#

	od;

	AppendTo(gsys,"\\hline");
	AppendTo(gsys,"\n\\end{tabular}\n\\end{center}\n");





	#print_multiplication_table := "true";
	
	if 	print_multiplication_table = "false" and
		print_basic_degree = "false"
	then

		AppendTo(gsys,"\n","\\end{document}","\n");

		CloseStream( gsys ); # this command will close gsys file

		return;
	
	fi;
#
#	calculate_nLH_number( G );
#
#	calculate_multiplication_table( G );
#
#	calculate_degrees_of_basic_maps( G );
#






#
#
#	AppendTo(gsys, "\n", "\\newpage", "\n");
#
#	AppendTo(gsys, "\n", "\\begin{center}", "\n");
#
#	AppendTo(gsys, "\n", "{\\bf The $n(L,H)$ Numbers for the Group $", G.LaTeXName, "$}\\\\", "\n");
#	AppendTo(gsys, "\\vskip.5cm", "\n");
#
#
#	AppendTo(gsys, "\\rput{90}(0,-9){", "\n");
#
#	AppendTo(gsys, "\\scalebox{ 0.6 }{ ","\n");
#
#	AppendTo(gsys, "\\begin{tabular}{|c||*{", Size(G.ccsl), "}{c|}}", "\n");
#
#	AppendTo(gsys, "\\hline", "\n");
#
#
#
#	AppendTo(gsys, "$n(L,H)$ ");
#	for i in [1..Size(G.ccsl)] do
##		AppendTo(gsys, " & $(", G.ccsl[i].LaTeXName, ")$ ");
#	od;
#	
#	AppendTo(gsys, " \\\\ " , "\n");
#
#	AppendTo(gsys, "\\hline\\hline", "\n");
#
#
#	for i in [1..Size(G.ccsl)] do
#		AppendTo(gsys, "$(", G.ccsl[i].LaTeXName, ")$ ");
#
#		for j in [1..Size(G.ccsl)] do
#			AppendTo(gsys, " & ", String(G.nLH_Table[i][j]));
#		od;
#
#		AppendTo(gsys, " \\\\ ", "\n");
#	od;
#
#	AppendTo(gsys, "\\hline", "\n");
#
#	AppendTo(gsys, "\n", "\\end{tabular}", "\n");
#
#	AppendTo(gsys, "}", "\n");
#
#	AppendTo(gsys, "}", "\n");
#
#	AppendTo(gsys, "\n", "\\end{center}", "\n");
#
#
#
#




	if 	print_multiplication_table = "true" 
	then


	rows_page := 20;
	columns_page := 5;
	
	left_column := 1;

	while left_column < Size(G.ccsl) do

	top_row := 1;

	while top_row < Size(G.ccsl) do

		AppendTo(gsys, "\n", "\\newpage", "\n");

		AppendTo(gsys, "\n", "\\begin{center}", "\n");

#	AppendTo(gsys, "\n", "{\\bf The Multiplicatin Table for the Burnside Ring $A(", G.LaTeXName, ")$}\\\\", "\n");
#	AppendTo(gsys, "\\vskip.5cm", "\n");

		AppendTo(gsys, "\\rput{90}(0,-9){", "\n");


		AppendTo(gsys, "\\scalebox{ 0.8 }{ ","\n");


		AppendTo(gsys, "\\begin{tabular}{|c||*{", Size(G.ccsl), "}{c|}}", "\n");

		AppendTo(gsys, "\\hline", "\n");

		AppendTo(gsys, "$*$ ");
		
		for i in [left_column..minimum(Size(G.ccsl), (left_column + columns_page - 1))] do
			AppendTo(gsys, " & $(", G.ccsl[i].LaTeXName, ")$ ");
		od;
		
		AppendTo(gsys, " \\\\ " , "\n");

		AppendTo(gsys, "\\hline\\hline", "\n");


		for i in [top_row..minimum(Size(G.ccsl), (top_row + rows_page - 1))] do
			AppendTo(gsys, "$(", G.ccsl[i].LaTeXName, ")$ ");

			for j in [left_column..minimum(Size(G.ccsl), (left_column + columns_page - 1))] do

				AppendTo(gsys, " & $ ");

				m := 0; # m is used to check whether this is the first item

				for k in [1..Size(G.ccsl)] do

					#Print("\n", G.Multiplication_Table[i][j][k] , "\n");

					if G.Multiplication_Table[i][j][k] <> 0 then
						if m = 0 then
							m := 1;
						else
							AppendTo(gsys, " + ");
						fi;

						if G.Multiplication_Table[i][j][k] <> 1 then
							AppendTo(gsys, G.Multiplication_Table[i][j][k]);
						fi;

						AppendTo(gsys, "(", G.ccsl[k].LaTeXName ,")");

					fi;

				od;

				AppendTo(gsys, " $ ");

			od;

			AppendTo(gsys, " \\\\ ", "\n");
		od;

		AppendTo(gsys, "\\hline", "\n");

		AppendTo(gsys, "\n", "\\end{tabular}", "\n");

		AppendTo(gsys, "}", "\n");

		AppendTo(gsys, "}", "\n");

		AppendTo(gsys, "\n", "\\end{center}", "\n");

	top_row := top_row + rows_page;

	Print("\ntop_row = ", top_row, "\n");

	od;
	
	left_column := left_column + columns_page;
	
	Print("\nleft_column = ", left_column, "\n");

	od;

	fi;
	




	if 	print_basic_degree = "false"
	then

		AppendTo(gsys,"\n","\\end{document}","\n");

		CloseStream( gsys ); # this command will close gsys file

		return;
	
	fi;




	for i in [1..Size(G.Representation_List)] do

		AppendTo( gsys, "\n", "\\newpage", "\n" );

		AppendTo( gsys, "\n", "\\noindent Representation ", String(i), "\n" );

		AppendTo( gsys, "\n", "$\\deg_{\\mathcal{V}_{", String(i), "}} $ = " );
		
		AppendTo( gsys, "$", G.Representation_List[i].LaTeXName, "$", "\n" );
		
		AppendTo( gsys, "\n", "Orbit Types: " );
		
		for j in [1..Size(G.Representation_List[i].degree_component_list)] do

			if j > 1 then
				AppendTo( gsys, " , " ); 
			fi;

			AppendTo( gsys, "$(", G.ccsl[ G.Representation_List[i].degree_component_list[j].group_id ].LaTeXName, ")$"	);	
		
		od;
		
		AppendTo( gsys, "\n" );








		#AppendTo(gsys,"\n","\\newpage","\n");

		
		for j in [1..Size(G.Representation_List[i].degree_component_list)] do

			if j = 1 then
				G.Representation_List[i].degree_component_list[j].y_position := 16; 
			else
				t := G.ccsl[ G.Representation_List[i].degree_component_list[j].group_id ].order ;
				
				if t = G.ccsl[ G.Representation_List[i].degree_component_list[j-1].group_id ].order
				then G.Representation_List[i].degree_component_list[j].y_position := G.Representation_List[i].degree_component_list[j-1].y_position;
				else
G.Representation_List[i].degree_component_list[j].y_position := G.Representation_List[i].degree_component_list[j-1].y_position - 2;
				fi;
				
			fi;


			if j = 1 then
				G.Representation_List[i].degree_component_list[j].x_position := 0; 
			else
				t := G.ccsl[ G.Representation_List[i].degree_component_list[j].group_id ].order ;
				
				if t = G.ccsl[ G.Representation_List[i].degree_component_list[j-1].group_id ].order
				then G.Representation_List[i].degree_component_list[j].x_position := G.Representation_List[i].degree_component_list[j-1].x_position + 2;
				else
G.Representation_List[i].degree_component_list[j].x_position := -2;
				fi;
				
			fi;
		
		od;




		AppendTo(gsys,"\n","\\begin{center} Representation Lattice\\end{center}","\n");
		AppendTo(gsys,"\\vglue16cm\\hskip5cm","\n");
		

		AppendTo(gsys,"\\scalebox{ 1.0 }{","\n");


		for ii in [1..Size(G.Representation_List[i].degree_component_list)] do
			for jj in [1..Size(G.Representation_List[i].degree_component_list)] do
				if ii = jj then
					continue;
				fi;

				# if jj is a subgroup of ii
				if IsSubgroup( G.ccsl[ G.Representation_List[i].degree_component_list[ii].group_id ].representative, G.ccsl[ G.Representation_List[i].degree_component_list[jj].group_id ].representative )
				then
					ll := true; # is it a 'true' subgroup? that is, is there other group between them?
					for kk in [1..Size(G.Representation_List[i].degree_component_list)] do
						if kk = ii or kk = jj then
							continue;
						fi;

						if	IsSubgroup( G.ccsl[ G.Representation_List[i].degree_component_list[ii].group_id ].representative, G.ccsl[ G.Representation_List[i].degree_component_list[kk].group_id ].representative ) and
							IsSubgroup( G.ccsl[ G.Representation_List[i].degree_component_list[kk].group_id ].representative, G.ccsl[ G.Representation_List[i].degree_component_list[jj].group_id ].representative ) then
							ll:= false;
							break;
						fi;
					od;
			
					if ll = true then
						AppendTo(gsys, "\\psline[linecolor=green](");
						AppendTo(gsys, G.Representation_List[i].degree_component_list[ii].x_position, ",");
						AppendTo(gsys, G.Representation_List[i].degree_component_list[ii].y_position-1, ".8)(");
						AppendTo(gsys, G.Representation_List[i].degree_component_list[jj].x_position, ",");
						AppendTo(gsys, G.Representation_List[i].degree_component_list[jj].y_position, ".2)");
						AppendTo(gsys, "\n");
					fi;
				fi;
			od;
		od;	



		AppendTo(gsys,"\n");

		for ii in [1..Size(G.Representation_List[i].degree_component_list)] do
			AppendTo(gsys,"\\rput(");
			AppendTo(gsys,String(G.Representation_List[i].degree_component_list[ii].x_position),",");
			AppendTo(gsys,String(G.Representation_List[i].degree_component_list[ii].y_position),")");
			AppendTo(gsys,"{$(", G.ccsl[ G.Representation_List[i].degree_component_list[ii].group_id ].LaTeXName, ")$}");
			AppendTo(gsys,"\n");
		od;	

		AppendTo(gsys,"}","\n"); # end of \scalebox{}{


	od;



	AppendTo(gsys,"\n","\\end{document}","\n");

	CloseStream( gsys ); # this command will close gsys file

	return;




end; # create_LaTeX_file := function()


















#######################################################
#                                                     #
#           Build Cyclic Group                        #
#                                                     #
#           return: G := Z_n                          #
#                                                     #
#######################################################


BuildCyclicGroup := function ( GroupOrder )


local 
	G,
	ccsl_element,
	gamma, 
	all_conjugacy_classes,
	conjugacy_class,
	n,
 	i, j;


	G := rec
	(
		IsProductGroup := false,
		group := undefined_value,
		ccsl := undefined_value, # ccsl means Conjugacy Classes Subgroups List
		LaTeXName := undefined_value,
		TextName := undefined_value
	);


	n := GroupOrder;

	gamma := ();
	
	for i in [1..n-1] do
		gamma := (i,i+1) * gamma;
	od;

	G.group := Group([gamma]);
	G.TextName := Concatenation("Z", String(n) );
	G.LaTeXName := Concatenation("\\bb{Z}_{", String(n), "}" ); 

	all_conjugacy_classes := ConjugacyClassesSubgroups(G.group);	

	G.ccsl := [];



	for conjugacy_class in all_conjugacy_classes do

		ccsl_element := rec
		(
			order := undefined_value,
			representative := undefined_value,
			LaTeXName := undefined_value
		);

		ccsl_element.representative := Representative( conjugacy_class );

		ccsl_element.order := Order( ccsl_element.representative );



		# decide the name of the conjugacy class
		ccsl_element.LaTeXName := Concatenation("\\bb{Z}_{",String(ccsl_element.order),"}");

		Add( G.ccsl, ccsl_element ); 

	od;

	return G;

end;








#######################################################
#                                                     #
#           Build V_4 Group                           #
#                                                     #
#           return: G := V_4                          #
#                                                     #
#######################################################


BuildV4Group := function ( )


local 
	G,
	ccsl_element,
	gamma, 
	all_conjugacy_classes,
	conjugacy_class,
	n,
 	i, j;


	G := rec
	(
		IsProductGroup := false,
		group := undefined_value,
		ccsl := undefined_value, # ccsl means Conjugacy Classes Subgroups List
		LaTeXName := undefined_value,
		TextName := undefined_value
	);


	G.group := Group( [(1,2)(3,4), (1,3)(2,4), (1,4)(2,3)] );
	G.TextName := "V4";
	G.LaTeXName := "V_4"; 

	all_conjugacy_classes := ConjugacyClassesSubgroups(G.group);	

	G.ccsl := [];



	for conjugacy_class in all_conjugacy_classes do

		ccsl_element := rec
		(
			order := undefined_value,
			representative := undefined_value,
			LaTeXName := undefined_value
		);

		ccsl_element.representative := Representative( conjugacy_class );

		ccsl_element.order := Order( ccsl_element.representative );



		# decide the name of the conjugacy class

		# decide the name of the conjugacy class
		if ccsl_element.order = 1 then ccsl_element.LaTeXName := "\\bb{Z}_1";
		elif ccsl_element.order = 4 then ccsl_element.LaTeXName := "V_4";
		elif ccsl_element.representative = Group((1,2)(3,4)) then ccsl_element.LaTeXName := "\\bb{Z}_2^1";
		elif ccsl_element.representative = Group((1,3)(2,4)) then ccsl_element.LaTeXName := "\\bb{Z}_2^2";
		elif ccsl_element.representative = Group((1,4)(2,3)) then ccsl_element.LaTeXName := "\\bb{Z}_2^3";
		
		else
		fi;

		Add( G.ccsl, ccsl_element ); 

	od;

	return G;

end;





#######################################################
#                                                     #
#           Build Dihedral Group                      #
#                                                     #
#           return: DihedralGroup( order*2 )          #
#                                                     #
#           i.e G := D_order                          #
#                                                     #
#######################################################


BuildDihedralGroup := function ( GroupOrder )


local 
	G,
	ccsl_element,
	gamma, kappa,
	all_conjugacy_classes,
	conjugacy_class,
	n,
	Is_Cyclic,
	HasKappa,
	HasKappagamma,
	reflection,
	 i, j;


	G := rec
	(
		IsProductGroup := false,
		group := undefined_value,
		ccsl := undefined_value, # ccsl means Conjugacy Classes Subgroups List
		LaTeXName := undefined_value,
		TextName := undefined_value
	);

	#G.ccsl := rec
	#(
	#	order := undefined_value,
	#	representative := undefined_value,
	#	LaTeXName := undefined_value
	#);



	n := GroupOrder;

	gamma := (1,2);
	kappa := (2,n);

	for i in [2..n-1] do
		gamma := (i,i+1) * gamma;
		if i+1<n-i+1 then 
			kappa := (i+1,n-i+1) * kappa; 
		fi;
	od;

	G.group := Group([gamma,kappa]);
	G.TextName := Concatenation("D", String(n) );
	G.LaTeXName := Concatenation("D_{", String(n), "}" ); 

	all_conjugacy_classes := ConjugacyClassesSubgroups(G.group);	

	G.ccsl := [];



	for conjugacy_class in all_conjugacy_classes do

		ccsl_element := rec
		(
			order := undefined_value,
			representative := undefined_value,
			LaTeXName := undefined_value
		);

		ccsl_element.representative := Representative( conjugacy_class );

		ccsl_element.order := Order( ccsl_element.representative );

		# check whether kappa*gamma^even inside the representative group
		HasKappa := false;
		reflection := kappa;
		for i in [1..n-1] do
			if reflection in ccsl_element.representative then
				HasKappa := true;
				break;
			fi;

			reflection := reflection * gamma * gamma;
		od;




		# check whether kappa*gamma^odd inside the representative group
		HasKappagamma := false;
		reflection := kappa * gamma;
		for i in [1..n-1] do
			if reflection in ccsl_element.representative then
				HasKappagamma := true;
				break;
			fi;

			reflection := reflection * gamma * gamma;
		od;
		


		# check whether it is cyclic
		if IsCyclic(ccsl_element.representative) then
			Is_Cyclic := true;
		else
			Is_Cyclic := false;
		fi;

		if HasKappa or HasKappagamma then
			Is_Cyclic := false;
		fi;



		# decide the name of the conjugacy class
		if Is_Cyclic then
			ccsl_element.LaTeXName := Concatenation("\\bb{Z}_{",String(ccsl_element.order),"}");
		else
			if HasKappa then
				ccsl_element.LaTeXName := Concatenation("D_{",String(ccsl_element.order/2),"}");
			else
				ccsl_element.LaTeXName := Concatenation("\\wt{D}_{",String(ccsl_element.order/2),"}");
			fi;
		fi;

		Add( G.ccsl, ccsl_element ); 

	od;

	return G;

end;





#######################################################
#                                                     #
#           Build Alternating Group                   #
#                                                     #
#           return: AlternatingGroup( n )             #
#                                                     #
#           i.e G := A_n                              #
#                                                     #
#######################################################


BuildAlternatingGroup := function ( n )


local 
	G,
	ccsl_element,
	all_conjugacy_classes,
	conjugacy_class,
	 i, j;


	if n <= 3 then return undefined_value; fi;

	#if n = 3 then return BuildDihedralGroup( 3 ); fi;

	G := rec
	(
		IsProductGroup := false,
		group := undefined_value,
		ccsl := undefined_value, # ccsl means Conjugacy Classes Subgroups List
		LaTeXName := undefined_value,
		TextName := undefined_value
	);



	G.group := AlternatingGroup( n );
	G.TextName := Concatenation("A", String(n) );
	G.LaTeXName := Concatenation("A_{", String(n), "}" ); 

	all_conjugacy_classes := ConjugacyClassesSubgroups(G.group);	

	G.ccsl := [];



	for conjugacy_class in all_conjugacy_classes do

		ccsl_element := rec
		(
			order := undefined_value,
			representative := undefined_value,
			LaTeXName := undefined_value
		);

		ccsl_element.representative := Representative( conjugacy_class );

		ccsl_element.order := Order( ccsl_element.representative );


		# decide the name of the conjugacy class
		if ccsl_element.order = 1 then ccsl_element.LaTeXName := "\\bb{Z}_1";
		elif ccsl_element.order = 2 then ccsl_element.LaTeXName := "\\bb{Z}_2";
		elif ccsl_element.order = 3 then ccsl_element.LaTeXName := "\\bb{Z}_3";
		elif ccsl_element.order = 4 then ccsl_element.LaTeXName := "V_4";
		elif ccsl_element.order = 12 then ccsl_element.LaTeXName := "A_4";
		
		elif ccsl_element.order = 5 then ccsl_element.LaTeXName := "\\bb{Z}_5";
		elif ccsl_element.order = 6 then ccsl_element.LaTeXName := "D_3";
		elif ccsl_element.order = 10 then ccsl_element.LaTeXName := "D_5";
		elif ccsl_element.order = 60 then ccsl_element.LaTeXName := "A_5";
		
		else
		fi;

		Add( G.ccsl, ccsl_element ); 

	od;

	if n = 4 or n = 5 then return G; fi;
	
	return undefined_value;

end;




#######################################################
#                                                     #
#           Build Symmetric Group                     #
#                                                     #
#           return: G = S_n                           #
#                                                     #
#######################################################


BuildSymmetricGroup := function ( n )


local 
	G,
	ccsl_element,
	all_conjugacy_classes,
	conjugacy_class,
	S4, S5, S6, S7,
	A4, A5, A6, A7,
	 i, j;


	if n <= 2 then return undefined_value; fi;

	if n = 3 then return BuildDihedralGroup( 3 ); fi;

	G := rec
	(
		IsProductGroup := false,
		group := undefined_value,
		ccsl := undefined_value, # ccsl means Conjugacy Classes Subgroups List
		LaTeXName := undefined_value,
		TextName := undefined_value
	);


	
	A4 := AlternatingGroup( 4 );
	A5 := AlternatingGroup( 5 );
		
	S4 := SymmetricGroup( 4 );
	S5 := SymmetricGroup( 5 );


	#G.group := ClosureGroup( Group((1,2)), AlternatingGroup( n ) );
	G.group := SymmetricGroup( n );
	G.TextName := Concatenation("S", String(n) );
	G.LaTeXName := Concatenation("S_{", String(n), "}" ); 

	all_conjugacy_classes := ConjugacyClassesSubgroups(G.group);	

	G.ccsl := [];



	for conjugacy_class in all_conjugacy_classes do

		ccsl_element := rec
		(
			order := undefined_value,
			representative := undefined_value,
			LaTeXName := undefined_value
		);

		ccsl_element.representative := Representative( conjugacy_class );

		ccsl_element.order := Order( ccsl_element.representative );

	
		# decide the name of the conjugacy class
		if ccsl_element.order = 1 then ccsl_element.LaTeXName := "\\bb{Z}_1";
		elif ccsl_element.order = 2 then ccsl_element.LaTeXName := "\\bb{Z}_2";
		elif ccsl_element.order = 3 then ccsl_element.LaTeXName := "\\bb{Z}_3";
		elif ccsl_element.order = 4 then ccsl_element.LaTeXName := "V_4";
		elif ccsl_element.order = 8 then ccsl_element.LaTeXName := "D_4";
		elif ccsl_element.order = 12 then ccsl_element.LaTeXName := "A_4";
		elif ccsl_element.order = 24 then ccsl_element.LaTeXName := "S_4";
		
		elif ccsl_element.order = 5 then ccsl_element.LaTeXName := "\\bb{Z}_5";
		elif ccsl_element.order = 6 then ccsl_element.LaTeXName := "D_3";
		elif ccsl_element.order = 10 then ccsl_element.LaTeXName := "D_5";
		elif ccsl_element.order = 60 then ccsl_element.LaTeXName := "A_5";

		elif ccsl_element.order = 20 then ccsl_element.LaTeXName := "F_{20}";
		elif ccsl_element.order = 120 then ccsl_element.LaTeXName := "S_5";
		
		else
		fi;
		
		if 	ccsl_element.order = 2 and
			not( IsSubgroup( A4, ccsl_element.representative ) ) 
		then ccsl_element.LaTeXName := "D_1";
		fi;



		if 	( ccsl_element.order = 4 ) and
			( not( IsCyclic( ccsl_element.representative ) ) ) and
			( not( IsSubgroup( A4, ccsl_element.representative ) ) ) 
		then
			ccsl_element.LaTeXName := "D_2";
		fi;

		if 	ccsl_element.order = 4 and
			IsCyclic( ccsl_element.representative )
		then
			ccsl_element.LaTeXName := "\\bb{Z}_4";
		fi;


#
#		if 	ccsl_element.order = 6 and
#			not( IsCyclic( ccsl_element.representative ) ) and
#			not( IsSubgroup( A5, ccsl_element.representative ) ) 
#		then
#			ccsl_element.LaTeXName := "\\wt{D}_3";
#		fi;
#
#		if 	ccsl_element.order = 6 and
#			IsCyclic( ccsl_element.representative )
#		then
#			ccsl_element.LaTeXName := "\\bb{Z}_6";
#		fi;
#		


		if 	ccsl_element.order = 12 and
			not( IsSubgroup( A4, ccsl_element.representative ) ) 
		then ccsl_element.LaTeXName := "D_6";
		fi;


		Add( G.ccsl, ccsl_element ); 

	od;

	if n = 4 or n = 5 then return G; fi;
	
	return undefined_value;

end;






#############################################################
#                                                           #
#               Build Amalga Group 	                        #
#                                                           #
#           return: Amalga Group of G1 x G2                 #
#                                                           #
#############################################################


BuildAmalgaGroup := function ( G1, G2 )

local 
	G,
	ccsl_element,
	PG, # PG := ProductGroup( G1, G2 );
	PGccsl, # ccsl of PG
	Generator_List,
	Inter_Quotient_List,
	Inter_Quotient_Group,
	Ama,
	sum,
	Embed1, Embed2,
	hom1, hom2,
	h1, h2, k1, k2, element1, element2, 
	 i, j, k, l, m, n, o, p, q, r, s, t, u, v, w, x, y, z;


	G := rec
	(
		IsProductGroup := true,
		G1 := G1,
		G2 := G2,
		group := Group(()),
		ccsl := [], # ccsl means Conjugacy Classes Subgroups List
		LaTeXName := undefined_value,
		TextName := undefined_value
	);

	PG := DirectProduct( G1.group, G2.group );
	PGccsl := ConjugacyClassesSubgroups( PG );

	
	Embed1 := Embedding( PG, 1 );
	Embed2 := Embedding( PG, 2 );
	
		
	G.G1.group := Image( Embed1, G1.group );
	G.G2.group := Image( Embed2, G2.group );
	
	for s in [1..Size(G2.ccsl)] do
		G.G2.ccsl[s].representative := Image( Embed2, G2.ccsl[s].representative ); 
	od;
	
	Inter_Quotient_List := Inter_Quo( G.G1.group, G.G2.group );

	Ama := [];

	for Inter_Quotient_Group in Inter_Quotient_List do
		Add( Ama , Ama_Pro( G.G1, G.G2, Inter_Quotient_Group ) );
	od;


	#for i in [1..Size(Ama)] do
  	#	for j in [1..Size(Ama[i])] do
   	#		Print("\nAma[", i, "][", j, "] = ", Ama[i][j] , "\n");
  	#	od;
	#od;

	sum := Sum_Ama( Ama );

	Print("\n", "Sum = ", sum, "\n");



	G.TextName := Concatenation( G1.TextName, "x", G2.TextName );
	G.LaTeXName := Concatenation( G1.LaTeXName, " \\tm ", G2.LaTeXName ); 

	#G.LaTeXName := StripLineBreakCharacters( G.LaTeXName );

	G.ccsl := [];

	for i in [1..Size(Ama)] do 
	for j in [1..Size(Ama[i])] do
	
	for p in [1..Size(Ama[i][j])] do

		ccsl_element := rec
		(
			order := undefined_value,
			conjugacyclass := undefined_value,
			representative := undefined_value,
			normalizer := undefined_value,
			normalizer_name := undefined_value,
			LaTeXName := undefined_value,
			KernelPhi := undefined_value,
			KernelPsi := undefined_value,
			HNumberOne := i,
			HNumberTwo := j
		);




		hom1 := Ama[i][j][p][1][2];
		hom2 := Ama[i][j][p][2][2];
		k1   := Ama[i][j][p][1][1];	
		k2   := Ama[i][j][p][2][1];	
					
	
		Generator_List := [];

		for element1 in k1 do
			for element2 in k2 do

				if Image( hom1, element1 ) = Image( hom2, element2 ) then
					Add( Generator_List, element1 * element2 );
				fi;
			od;
		od;


		t := Group( Generator_List );

		
						
		for k in [1..Size(PGccsl)] do
			if IsConjugate( PG, Representative(PGccsl[k]), t ) then
				ccsl_element.representative := Representative(PGccsl[k]);
			
				break;
			fi;
		od;


		G.group := ClosureGroup( G.group, ccsl_element.representative );

		ccsl_element.order := Order( ccsl_element.representative );

		if i = 1 then
			ccsl_element.LaTeXName := Concatenation( Ama[i][j][p][5], "  \\tm_{" , Ama[i][j][p][4], "} ", Ama[i][j][p][6] );
		else
			ccsl_element.LaTeXName := Concatenation( Ama[i][j][p][5], " { \\, ^{\\vp } \\tm_{" , Ama[i][j][p][4], "} ^ \\psi } ", Ama[i][j][p][6] );
		fi;

		#ccsl_element.LaTeXName := StripLineBreakCharacters( ccsl_element.LaTeXName );


		ccsl_element.KernelPhi := HSymbol( G.G1, Kernel( Ama[i][j][p][1][2] ) );

		ccsl_element.KernelPsi := HSymbol( G.G2, Kernel( Ama[i][j][p][2][2] ) );

		Add( G.ccsl, ccsl_element ); 

	od;
	od; 
	od;
	
	
	for i in [1..Size(G.ccsl)] do
		G.ccsl[i].normalizer := Normalizer( G.group, G.ccsl[i].representative ); 
		G.ccsl[i].right_transversal_for_normalizer :=
			RightTransversal( G.group, G.ccsl[i].normalizer );
	od;


	# look for the name for normalizer
	for i in [1..Size(G.ccsl)] do
		for j in [1..Size(G.ccsl)] do
			if IsConjugate( G.group, G.ccsl[i].normalizer, G.ccsl[j].representative ) then
				G.ccsl[i].normalizer_name := Concatenation( G.ccsl[j].LaTeXName, " , \\text{\\ft \\; No.", String(j), " } " );
				
				#G.ccsl[i].normalizer_name := StripLineBreakCharacters( G.ccsl[i].normalizer_name );			
				
				break;
			fi;
		od;
	od;



	# calculate Weyl group order
	for i in [1..Size(G.ccsl)] do
		G.ccsl[i].order_Weyl_group := Order( G.ccsl[i].normalizer ) / Order( G.ccsl[i].representative );
	od;



	for i in [1..Size(G.ccsl)] do
		G.ccsl[i].Original_Place := i;
	od;



	for i in [1..Size(G.ccsl)] do
		for j in [i..Size(G.ccsl)] do
			if G.ccsl[i].order > G.ccsl[j].order then
				t := G.ccsl[i];
				G.ccsl[i] := G.ccsl[j];
				G.ccsl[j] := t;
			fi;
		od;
	od;

	
	i := Size(G.ccsl);
	
	while G.ccsl[i].representative <> G.group do i := i - 1; od;

	t := G.ccsl[i];
	G.ccsl[i] := G.ccsl[Size(G.ccsl)];
	G.ccsl[Size(G.ccsl)] := t;
		
		
	return G;

end;



